/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.router;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.eingang.common.formdata.AntragstellerTestFactory;
import de.ozgcloud.eingang.common.formdata.FormDataTestFactory;
import de.ozgcloud.eingang.common.formdata.IncomingFileTestFactory;
import de.ozgcloud.eingang.common.formdata.ZustaendigeStelleTestFactory;
import de.ozgcloud.vorgang.common.grpc.GrpcFormDataMapper;
import de.ozgcloud.vorgang.vorgang.GrpcAntragsteller;
import de.ozgcloud.vorgang.vorgang.GrpcEingang;
import de.ozgcloud.vorgang.vorgang.GrpcFormData;
import de.ozgcloud.vorgang.vorgang.GrpcServiceKonto;
import de.ozgcloud.vorgang.vorgang.GrpcZustaendigeStelle;

class GrpcEingangMapperTest {

	@InjectMocks
	private GrpcEingangMapper mapper = Mappers.getMapper(GrpcEingangMapper.class);
	@Mock
	private GrpcFormDataMapper grpcFormDataMapper;
	@Mock
	private ServiceKontoMapper serviceKontoMapper;

	@DisplayName("To antragsteller")
	@Nested
	class TestToAntragsteller {

		@Test
		void antragstellerShouldBeMapped() {
			var mapped = toAntragsteller();

			assertThat(mapped.getPostfachId()).isEqualTo(AntragstellerTestFactory.POSTFACH_ID);
			assertThat(mapped.getVorname()).isEqualTo(AntragstellerTestFactory.VORNAME);
			assertThat(mapped.getTelefon()).isEqualTo(AntragstellerTestFactory.TELEFON);

		}

		@Test
		void shouldMapFirmaName() {
			var mapped = mapper.toAntragsteller(AntragstellerTestFactory.createFirm());

			assertThat(mapped.getFirmaName()).isEqualTo(AntragstellerTestFactory.FIRMA_NAME);
		}

		private GrpcAntragsteller toAntragsteller() {
			return mapper.toAntragsteller(AntragstellerTestFactory.create());
		}
	}

	@DisplayName("To zustaendigeStelle")
	@Nested
	class TestToZustaendigeStelle {

		@Test
		void eingangShouldHaveZustaendigeStelle() {
			var zustaendigeStelle = toZustaendigeStelle();

			assertThat(zustaendigeStelle).isNotNull();
			assertThat(zustaendigeStelle.getOrganisationseinheitenId()).isEqualTo(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEIT_ID);
			assertThat(zustaendigeStelle.getEmail()).isEqualTo(ZustaendigeStelleTestFactory.EMAIL);
		}

		private GrpcZustaendigeStelle toZustaendigeStelle() {
			return mapper.toZustaendigeStelle(ZustaendigeStelleTestFactory.create());
		}
	}

	@DisplayName("To eingang")
	@Nested
	class TestToEingang {

		@BeforeEach
		void mockMapperReturnValues() {
			when(grpcFormDataMapper.mapToFormData(anyMap()))
					.thenReturn(GrpcFormData.newBuilder().addField(GrpcFormFieldTestFactory.create()).build());
			when(serviceKontoMapper.toServiceKonto(any())).thenReturn(GrpcServiceKonto.newBuilder().build());
		}

		@Nested
		@DisplayName("Mapped Antragsteller")
		class TestMappingAntragsteller {

			@Test
			void dataShouldBeMapped() {
				var antragsteller = toEingang().getAntragsteller();

				assertThat(antragsteller.getOtherData().getFieldList()).hasSize(1);
				assertThat(antragsteller.getOtherData().getField(0).getName()).isEqualTo(GrpcFormFieldTestFactory.TEST_NAME);
				assertThat(antragsteller.getOtherData().getField(0).getValue()).isEqualTo(GrpcFormFieldTestFactory.TEST_VALUE);
			}
		}

		@Nested
		@DisplayName("Test mapped Attachments")
		class TestAttachments {

			@Test
			void validateNumberOfAttachments() {
				var eingang = toEingang();

				assertThat(eingang.getNumberOfAttachments()).isEqualTo(2);
			}

			@Test
			void validateNumberOfAttachmentGroups() {
				var eingang = toEingang();

				assertThat(eingang.getAttachmentsCount()).isEqualTo(2);
			}

			@Test
			void validateGroup1AttachmentCount() {
				var eingang = toEingang();

				assertThat(eingang.getAttachmentsList().get(0).getFilesCount()).isEqualTo(1);
			}

			@Test
			void validateGroup1Attachment() {
				var eingang = toEingang();

				var attachment = eingang.getAttachmentsList().get(0).getFilesList().get(0);
				assertThat(attachment.getId()).isEqualTo(IncomingFileTestFactory.ID);
				assertThat(attachment.getVendorId()).isEqualTo(IncomingFileTestFactory.VENDOR_ID);
				assertThat(attachment.getName()).isEqualTo(IncomingFileTestFactory.NAME);
				assertThat(attachment.getContentType()).isEqualTo(IncomingFileTestFactory.CONTENT_TYPE);
				assertThat(attachment.getContent()).isEmpty();
			}

			@Test
			void validateGroup2AttachmentCount() {
				var eingang = toEingang();

				assertThat(eingang.getAttachmentsList().get(1).getFilesCount()).isEqualTo(1);
			}

			@Test
			void validateGroup2Attachment() {
				var eingang = toEingang();

				var attachment = eingang.getAttachmentsList().get(1).getFilesList().get(0);
				assertThat(attachment.getId()).isEqualTo(IncomingFileTestFactory.ID);
				assertThat(attachment.getVendorId()).isEqualTo(IncomingFileTestFactory.VENDOR_ID);
				assertThat(attachment.getName()).isEqualTo(IncomingFileTestFactory.NAME);
				assertThat(attachment.getContentType()).isEqualTo(IncomingFileTestFactory.CONTENT_TYPE);
				assertThat(attachment.getContent()).isEmpty();
			}
		}

		@Nested
		@DisplayName("Test mapped Representations")
		class TestRepresentations {

			@Test
			void testRepresentations() {
				var eingang = toEingang();

				assertThat(eingang.getRepresentationsCount()).isEqualTo(1);
				var representation = eingang.getRepresentationsList().get(0);
				assertThat(representation.getId()).isEqualTo(IncomingFileTestFactory.ID);
				assertThat(representation.getVendorId()).isEqualTo(IncomingFileTestFactory.VENDOR_ID);
				assertThat(representation.getName()).isEqualTo(IncomingFileTestFactory.NAME);
				assertThat(representation.getContentType()).isEqualTo(IncomingFileTestFactory.CONTENT_TYPE);
				assertThat(representation.getContent()).isEmpty();
			}
		}

		@Nested
		@DisplayName("Test mapped Header")
		class TestHeader {
			@Test
			void shouldMapAllFields() {
				var header = toEingang().getHeader();

				assertThat(header).usingRecursiveAssertion().isEqualTo(GrpcEingangHeaderTestFactory.create());

			}
		}

		private GrpcEingang toEingang() {
			return mapper.toEingang(FormDataTestFactory.create());
		}
	}
}